from uuid import UUID

import fastapi
from starlette import status

from .schemas import FileBaseSchema, FileGetSchema
from .ctrl import FileController
from ..dependencies import authenticate_user
from db.repositories import UserRepository, FolderRepository, FileRepository
from exceptions import common as common_exc, http as http_exc


router = fastapi.APIRouter(prefix='/files', tags=['File'])
ctrl = FileController()
user_repo = UserRepository()
folder_repo = FolderRepository()
file_repo = FileRepository()


@router.get('')
async def get_files(username: str = fastapi.Depends(authenticate_user)):
    user = await user_repo.get_user_by_username(username)
    files = await user.user_files
    return [FileGetSchema.model_validate(file) for file in files]


@router.get('/{id}')
async def get_file(id: UUID, username: str = fastapi.Depends(authenticate_user)):
    user = await user_repo.get_user_by_username(username)
    file = await file_repo.get(id)
    if user.id != file.user_id:
        raise fastapi.HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="You do not have permission to access this file",
        )
    file = await ctrl.get(id)
    return FileGetSchema.model_validate(file)


@router.post('')
async def create_file(
    body: FileBaseSchema,
    username: str = fastapi.Depends(authenticate_user)
):
    name = body.name
    content = body.content
    try:
        folder = await folder_repo.get(id=body.folder)
    except common_exc.NotFoundException:
        raise http_exc.HTTPBadRequestException(detail='Object folder does not exist')
    user = await user_repo.get_user_by_username(username)
    file = await ctrl.create(name=name, content=content, folder=folder, user=user)
    return FileGetSchema.model_validate(file)


@router.patch('/{id}')
async def update_file(
    id: UUID, body: FileBaseSchema,
    username: str = fastapi.Depends(authenticate_user)
):
    user = await user_repo.get_user_by_username(username)
    file = await file_repo.get(id)
    if user.id != file.user_id:
        raise fastapi.HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="You do not have permission to access this file",
        )
    try:
        folder = await folder_repo.get(id=body.folder)
    except common_exc.NotFoundException:
        raise http_exc.HTTPBadRequestException(detail='Object folder does not exist')

    file = await ctrl.update(id, name=body.name, content=body.content, folder=folder)
    return FileGetSchema.model_validate(file)


@router.delete('/{id}')
async def delete_file(
    id: UUID,
    username: str = fastapi.Depends(authenticate_user)
):
    user = await user_repo.get_user_by_username(username)
    file = await file_repo.get(id)
    if user.id != file.user_id:
        raise fastapi.HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="You do not have permission to access this file",
        )
    return await ctrl.delete(id)
