from uuid import UUID

import fastapi
from starlette import status

from .schemas import FolderBaseSchema, FolderSchema
from .ctrl import FolderController
from ..dependencies import authenticate_user
from db.repositories import UserRepository, FolderRepository

from ..files.schemas import FileGetSchema

router = fastapi.APIRouter(prefix='/folders', tags=['Folder'])
ctrl = FolderController()
user_repo = UserRepository()
folder_repo = FolderRepository()


@router.get('')
async def get_folders(username: str = fastapi.Depends(authenticate_user)):
    user = await user_repo.get_user_by_username(username)
    folders = await user.user_folders
    return [FolderSchema.model_validate(folder) for folder in folders]


@router.get('/{id}')
async def get_folder(id: UUID, username: str = fastapi.Depends(authenticate_user)):
    user = await user_repo.get_user_by_username(username)
    folder = await folder_repo.get(id)
    if user.id != folder.user_id:
        raise fastapi.HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="You do not have permission to access this file",
        )
    files_in_folder = await folder.folder_files.all()
    return {
        "folder": FolderSchema.model_validate(folder),
        "files": [FileGetSchema.model_validate(file) for file in files_in_folder]
    }


@router.post('')
async def create_folder(
    body: FolderBaseSchema,
    username: str = fastapi.Depends(authenticate_user)
):
    name = body.name
    user = await user_repo.get_user_by_username(username)
    return await ctrl.create(name=name, user=user)


@router.patch('/{id}')
async def update_folder(
    id: UUID, body: FolderBaseSchema,
    username: str = fastapi.Depends(authenticate_user)
):
    user = await user_repo.get_user_by_username(username)
    folder = await folder_repo.get(id)
    if user.id != folder.user_id:
        raise fastapi.HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="You do not have permission to access this file",
        )
    folder = await ctrl.update(id, **body.model_dump(exclude_none=True))
    return FolderSchema.model_validate(folder)


@router.delete('/{id}')
async def delete_folder(
    id: UUID,
    username: str = fastapi.Depends(authenticate_user)
):
    user = await user_repo.get_user_by_username(username)
    folder = await folder_repo.get(id)
    if user.id != folder.user_id:
        raise fastapi.HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="You do not have permission to access this file",
        )
    return await ctrl.delete(id)
