from uuid import UUID
from datetime import timedelta

import fastapi as fa
from fastapi.security import OAuth2PasswordRequestForm
from starlette import status

from .schemas import UserCreateSchema, UserGetSchema, UserUpdateSchema
from .ctrl import UserController
from .services import UserService
import settings
from db.repositories import UserRepository


router = fa.APIRouter(prefix='/users', tags=['User'])
token_router = fa.APIRouter(tags=['AUTH'])
ctrl = UserController()
user_service = UserService()
user_repository = UserRepository()


@token_router.post('/token')
async def login(form_data: OAuth2PasswordRequestForm = fa.Depends()):
    user = await user_service.authenticate_user(form_data.username, form_data.password)

    if not user:
        raise fa.HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Неверное имя пользователя или пароль",
            headers={"WWW-Authenticate": "Bearer"},
        )

    access_token_expires = timedelta(minutes=int(settings.access_token_expire_minutes))
    refresh_token_expires = timedelta(days=int(settings.refresh_token_expire_minutes))

    access_token = await user_service.create_access_token(
        data={"sub": user.username}, expires_delta=access_token_expires
    )
    refresh_token = await user_service.create_refresh_token(
        data={"sub": user.username}, expires_delta=refresh_token_expires
    )

    return {"access_token": access_token, "refresh_token": refresh_token, "token_type": "bearer"}


@router.get('/get',  response_model=UserGetSchema)
async def get_user(token: str):
    return await ctrl.get(token)


@router.post('')
async def create_user(body: UserCreateSchema):
    user = await ctrl.create(**body.model_dump(exclude_none=True))
    return UserGetSchema.model_validate(user)


@router.patch('/{id}')
async def update_user(
    id: UUID, body: UserUpdateSchema,
    token: str
):
    current_user = await user_repository.get_current_user(token)
    if current_user.id != id:
        raise fa.HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="You are not allowed to update this user",
        )
    user = await ctrl.update(id, **body.model_dump(exclude_none=True))
    return UserGetSchema.model_validate(user)


@router.delete('/{id}')
async def delete_user(
    id: UUID,
    token: str
):
    current_user = await user_repository.get_current_user(token)
    if current_user.id != id:
        raise fa.HTTPException(
            status_code=status.HTTP_403_FORBIDDEN,
            detail="You are not allowed to delete this user",
        )
    return await ctrl.delete(id)
