from uuid import UUID
from datetime import datetime, timedelta

import fastapi as fa
from jose import jwt
from passlib.context import CryptContext
from starlette import status

from db.repositories import UserRepository
import settings
from db.models import User


class UserService:
    user_repository = UserRepository()
    pwd_context = CryptContext(schemes=['bcrypt'], deprecated='auto')

    async def get_user(self, token: str):
        return await self.user_repository.get_current_user(token)

    async def create_user(self, **kwargs):
        return await self.user_repository.create(**kwargs)

    async def update_user(self, id: UUID, **kwargs):
        return await self.user_repository.update(id, **kwargs)

    async def delete_user(self, id: UUID):
        return await self.user_repository.delete(id)

    async def verify_password(self, plain_password, hashed_password):
        return self.pwd_context.verify(plain_password, hashed_password)

    async def authenticate_user(self, username: str, password: str) -> User:
        user = await self.user_repository.get_user_by_username(username)

        if user and await self.verify_password(password, user.password):
            return user

        return None

    async def create_access_token(self, data: dict, expires_delta: timedelta):
        to_encode = data.copy()
        expire = datetime.utcnow() + expires_delta
        to_encode.update({"exp": expire})
        return jwt.encode(to_encode, settings.secret_key, algorithm=settings.algorithm)

    async def create_refresh_token(self, data: dict, expires_delta: timedelta):
        to_encode = data.copy()
        expire = datetime.utcnow() + expires_delta
        to_encode.update({"exp": expire})
        return jwt.encode(to_encode, settings.secret_key, algorithm=settings.algorithm)

    async def get_object_user(self, username: str):
        user = self.user_repository.get(username == username).first()
        if not user:
            raise fa.HTTPException(
                status_code=status.HTTP_401_UNAUTHORIZED,
                detail="Not authorized",
                headers={"WWW-Authenticate": "Bearer"},
            )
        return user
