from uuid import UUID

import tortoise
from passlib.context import CryptContext

from .models import User, Folder, File
from exceptions import common as common_exc
from api.dependencies import authenticate_user


class BaseRepository:
    model: tortoise.Model

    async def get_list(self, **kwargs) -> list[tortoise.Model]:
        return await self.model.filter(**kwargs)

    async def get(self, id: UUID) -> tortoise.Model:
        try:
            return await self.model.get(id=id)

        except tortoise.exceptions.DoesNotExist as e:
            raise common_exc.NotFoundException(str(e))

    async def create(self, **kwargs) -> tortoise.Model:
        try:
            return await self.model.create(**kwargs)

        except tortoise.exceptions.IntegrityError as e:
            raise common_exc.CreateException(str(e))

    async def update(self, id: UUID, **kwargs) -> tortoise.Model:
        try:
            instance = await self.get(id=id)
            await instance.update_from_dict(kwargs).save()

            return instance

        except tortoise.exceptions.IntegrityError as e:
            raise common_exc.UpdateException(str(e))

    async def delete(self, id: UUID) -> None:
        try:
            instance = await self.get(id=id)
            await instance.delete()

        except tortoise.exceptions.IntegrityError as e:
            raise common_exc.DeleteException(str(e))


class UserRepository(BaseRepository):
    model = User

    @staticmethod
    def __get_password_hash(password: str) -> str:
        pwd_context = CryptContext(schemes=["bcrypt"], deprecated="auto")
        return pwd_context.hash(password)

    async def get_current_user(self, token):
        return await self.get_user_by_username(authenticate_user(token))

    async def create(self, **kwargs):
        kwargs['password'] = self.__get_password_hash(kwargs['password'])
        return await super().create(**kwargs)

    async def get_user_by_username(self, username: str) -> User:
        return await self.model.get(username=username)


class FolderRepository(BaseRepository):
    model = Folder


class FileRepository(BaseRepository):
    model = File
